%%%%%%%%%%%
%% Forecast ...
%%%%%%%%%%%
close all; clear all

readmodel;

% Loads the model structure from the file 'model.mat' (the file 'model.mat'
% is saved by the 'readmodel' program) into the object 'm' and the historical
% data from the file 'history.mat' (the file 'history.mat' is saved by the 
% 'makedata' program) into the object 'h'
%m = loadstruct('model.mat');
h = loadstruct('history.mat');

% Defines the time frame of the forecast
% Don't forget to change the time frame depending on you data and forecast
% period
startfcast = qq(2011,1);
endfcast = qq(2021,4);
fcastrange = startfcast:endfcast;

% Shock to risk premium
% h.shk_prem = tseries(startfcast:startfcast+1,1);

% Shock to aggregate supply (e.g., increase in indirect taxes)
% h.shk_dot_cpi = tseries(startfcast,1.5); 

% Shock to aggregate demand (e.g., sharp cyclical slowdown)
% h.shk_lgdp_gap = tseries(startfcast:startfcast+1,-0.75); 

% h.shk_lx_gdp_gap = tseries(startfcast:startfcast,-3);

% Command 'simulate' simulates the model 'm' based on the database 'h' over
% the forecast range 'fcastrange'. Results are written into the object 's'
s = simulate(m,h,fcastrange,'anticipate',false);

%% In the forecast simulation variables can be set as numbers (for instance
% for external development as opposed to automatically used AR(1) process). 
% These preset variables are in the historical database and they require 
% creation of a 'plan' for this variable. The command 'plan' creates plan 
% for model 'm' over the forecast horizon in the object 'simplan'. These
% variables must be marked using the command 'exogenize', while the residual
% used for equalizing the model equation for the fixed variable must be marked using
% command 'endogenize'. Note that any variable that is fixed must have residual term
% as a part of its equation. Finally, the forecast is simulated in a roughly 
% same way as without the fixing. See example below where external inflation 
% and external interest rates are taken, say, from IMF's WEO or Consensus
% Forecast:

% %Defining foreign inflation
% h.dot_x_cpi(startfcast:startfcast+1) = 4;
% % 
% % Fixing foreign inflation 
% simplan = plan(m,startfcast:startfcast+1);
% simplan = exogenize(simplan,{'dot_x_cpi'},startfcast:startfcast+1);
% simplan = endogenize(simplan,{'shk_dot_x_cpi'},startfcast:startfcast+1);
% s = simulate(m,h,fcastrange,'plan',simplan);

% %Defining domestic policy rate
% h.rn(startfcast:startfcast) = 1.2;
%  
% %Fixing the policy rate for one quarter
% simplan = plan(m,startfcast:startfcast);
% simplan = exogenize(simplan,{'rn'},startfcast:startfcast);
% simplan = endogenize(simplan,{'shk_rn'},startfcast:startfcast);
% s = simulate(m,h,fcastrange,'plan',simplan);

% % Defining foreign policy rate
% h.x_rn(startfcast:startfcast+1) = 3;
%  
% %Fixing foreign interest rates
% simplan = plan(m,startfcast:startfcast+1);
% simplan = exogenize(simplan,{'x_rn'},startfcast:startfcast+1);
% simplan = endogenize(simplan,{'shk_x_rn'},startfcast:startfcast+1);
% s = simulate(m,h,fcastrange,'plan',simplan);

% % Defining foreign output gap
% h.lx_gdp_gap(startfcast:startfcast+1) = -3;
%  
% %Fixing foreign output gap
% simplan = plan(m,startfcast:startfcast+1);
% simplan = exogenize(simplan,{'lx_gdp_gap'},startfcast:startfcast+1);
% simplan = endogenize(simplan,{'shk_lx_gdp_gap'},startfcast:startfcast+1);
% s = simulate(m,h,fcastrange,'plan',simplan);

% Command 'dboverlay' puts together the historical database 'h' and the
% results of the simulation saved in object 's'. Single database 's' is
% created
s = dboverlay(h,s);

% Results are saved in file 'fcastdata.mat'  
savestruct('fcastdata.mat', s);

%% Graphs and Tables
% Prepares the forecast report: graphs and tables in the PostScript format
% See the ned of this file to change the output from PostScript to .pdf
Tablerng = startfcast-3:startfcast+7;
Plotrng = startfcast-3:startfcast+11;
Histrng = startfcast-3:startfcast-1;

% Specify country and units for exchange rate
country = 'Czech Republic';
exchange = 'CZK/EUR';

Heading = country;

x = report('heading',Heading,'dateformat','YYYY:P','color', true);
x = newpage(x);

x = opengraph(x);
  x = tag(x,'Forecast - Main Indicators','fontsize','large');
  
  subplot(2,3,1); hold on;
  plot(Plotrng,s.dot4_cpi,'linewidth',2,{'color'},{'r'},{'linestyle'},{'-'}); title('Inflation'); 
  hold on
  plot(Plotrng,s.dot_cpi,'linewidth',2,{'color'},{'b'},{'linestyle'},{'--'}); 
  hold on
  plot(Plotrng,[s.target],'linewidth',2,{'color'},{'g'},{'linestyle'},{'-.'}); 
  legend('y-o-y','q-o-q','Target','location','southeast');
  grid on; box off;
  grayarea(Histrng);
  
  subplot(2,3,2); hold on;
  plot(Plotrng,s.rn,'linewidth',2); title('Nominal Interest Rates'); 
  grid on; box off;
  grayarea(Histrng);
  
  subplot(2,3,3); hold on;
  plot(Plotrng,exp(s.ls/100),'linewidth',2); title('Exchange Rate(CZK/EUR)'); 
  grid on; box off;
  grayarea(Histrng);

  subplot(2,3,4); hold on;
  plot(Plotrng,[s.dot_s],'linewidth',2); title('Exchange Rate (q-o-q)'); 
  %legend(exchange,'Target','location','southeast'); 
  grid on; box off;
  grayarea(Histrng);
    
  subplot(2,3,5); hold on;
  plot(Plotrng,[s.lgdp_gap s.cum_gap],'linewidth',2,{'color'},{'b';'r'},{'linestyle'},{'-';'-.'}); title('Output Gap'); 
  legend('Output Gap','Incremental Gap','location','southeast'); 
  grid on; box off;
  grayarea(Histrng);

  subplot(2,3,6); hold on;
  plot(Plotrng,[s.rr_gap s.lz_gap],'linewidth',2,{'color'},{'b';'r'},{'linestyle'},{'-';'--'}); title('Real Interest Rate and Real Exchange Rate Gap'); 
  legend('RIR Gap','RER Gap','location','southeast'); 
  grid on; box off;
  grayarea(Histrng);

x = getgraph(x);

x = newpage(x);

x = begintable(x,'range',Tablerng,'divider',Histrng(end),'fontsize', 'small');
x = tag(x,'Forecast - Main Indicators','fontsize','normal');
    x = intertext(x, '', 'bold', true);
    
    % print CPI
    x = row(x,s.dot4_cpi,    'text', 'CPI',                    'unit','% y-o-y');
    x = row(x,s.dot_cpi,     'text', ' ',                      'unit','% q-o-q (annual.)');
    x = row(x,s.target,     'text', 'Target',                  'unit','% y-o-y');
    
    % print GDP
    x = row(x,s.dot4_gdp,    'text', 'Real GDP',               'unit','% y-o-y');
    x = row(x,s.dot_gdp,     'text', '',                       'unit','% q-o-q (annual.)');
   
    x = intertext(x, '', 'bold', true);
    
    % print IR
    x = intertext(x, 'Nominal Interest Rate', 'bold', true);
    x = row(x,s.rn,    'text', 'Market Rate',                    'unit','%');
    x = row(x,s.rn_neutral,    'text', 'Policy Neutral Rate',    'unit','%');
    
    x = intertext(x, '', 'bold', true);
    
    % print ER
    x = intertext(x, 'Nominal Exchange Rate', 'bold', true);
    x = row(x,diff(s.ls, -4),  'text', exchange,                 'unit','% y-o-y');
    x = row(x,s.dot_s,  'text', '',                                'unit','% q-o-q (annual.)');
    x = row(x,exp(s.ls/100),  'text', '',                         'unit','level');
    x = row(x,s.prem,  'text', 'Risk Premium',            'unit','% of Exchange Rate');
    x = intertext(x, '', 'bold', true);

    % print Monetary conditions
    x = intertext(x, 'Monetary Conditions', 'bold', true);
    x = row(x,s.mci,  'text', 'Real Monetary Conditions',        'unit','%');
    x = row(x,s.rr_gap,  'text', 'Real Interest Rate Gap',        'unit','%');
    x = row(x,s.lz_gap,  'text', 'Real Exchange Rate Gap',         'unit','%');
    x = intertext(x, '', 'bold', true);
    
    %print Trends
    x = intertext(x, 'Supply Side Assumptions', 'bold', true);
    x = row(x,s.dot_z_eq,  'text', 'Real Exchange Rate Trend',      'unit','% q-o-q (annual.)');
    x = row(x,s.rr_eq,  'text', 'Real Interest Rate Trend',        'unit','%');
    x = row(x,s.dot_gdp_eq,  'text', 'Potential GDP',               'unit','% q-o-q (annual.)');
    x = intertext(x, '', 'bold', true);    
    
x = endtable(x);   
    
x = newpage(x);
x = begintable(x,'range',Tablerng,'divider',Histrng(end),'fontsize', 'small');
    x = tag(x,'Forecast - Decomposition','fontsize','small');
    
    % print External Development
        
    x = intertext(x, 'External Development', 'bold', true);
    x = row(x,s.lx_gdp_gap,  'text', 'Foreign Output Gap',      'unit','%');
    x = row(x,s.dot_x_cpi,  'text', 'Foreign Inflation',          'unit','% q-o-q (annual.)');
    x = row(x,s.x_rn,  'text', 'Foreign Nominal Interest Rate',       'unit','%');
    x = intertext(x, '', 'bold', true);

    % print Output Decomposition
    x = intertext(x, 'Output Gap Decomposition', 'bold', true);
    x = row(x,s.lgdp_gap,  'text', 'Output Gap',                    'unit','%');
    x = row(x,s.a1*s.lgdp_gap{-1},  'text', 'Output Gap Lag',        'unit','pp');
    x = row(x,-s.a2*s.mci,  'text', 'Real Monetary Conditions',       'unit','pp');
    x = row(x,s.a3*s.lx_gdp_gap,  'text', 'Foreign Output Gap',      'unit','pp');
    x = intertext(x, '', 'bold', true);
 
    % print Core Decomposition
    x = intertext(x, 'Inflation Factor Decomposition', 'bold', true);
    x = row(x,s.dot_cpi,  'text', 'Inflation',                     'unit','% q-o-q (annual.)');
    x = row(x,s.b1*s.dot_cpi{-1},  'text', 'Inflation Lag',         'unit','pp');
    x = row(x,(1-s.b1)*s.dot_cpi{+1},  'text', 'Inflation Expectation', 'unit','pp');
    x = row(x,s.b2*s.rmc,  'text', 'Real Marginal Costs',         'unit','pp');
    x = intertext(x, '', 'bold', true);
    
    % print Interest Rate Decomposition
    x = intertext(x, 'Interest Rate Decomposition', 'bold', true);
    x = row(x,s.rn,  'text', 'Interest Rate',                            'unit','%');     
    x = row(x,s.f1*s.rn{-1},  'text', 'Interest Rate Lag',                'unit','pp'); 
    x = row(x,(1-s.f1)*s.rn_neutral,  'text', 'Policy Neutral Interest Rate', 'unit','pp');
    x = row(x,(1-s.f1)*(s.f2*(s.dot4_cpi{+4}-s.target{+4})+s.f3*s.lgdp_gap),  'text', 'Policy Rule',       'unit','pp');
    x = intertext(x, '', 'bold', true);
   
x = endtable(x);

%% The default is set to generate a PostScript output, but it can be
%% changed to a .pdf (Acrobat) file by:
compile(x,'Forecast.pdf');
compile(x,'Forecast.ps','display',false);